package cn.reghao.tnb.content.app.geo.db.mapper;

import cn.reghao.jutil.jdk.db.BaseMapper;
import cn.reghao.tnb.content.app.geo.model.po.CameraPhoto;
import cn.reghao.tnb.content.app.geo.model.vo.MapMarker;
import cn.reghao.tnb.content.app.geo.model.vo.MarkerInfo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author reghao
 * @date 2024-09-07 12:01:54
 */
@Mapper
public interface CameraPhotoMapper extends BaseMapper<CameraPhoto> {
    void updateSetAlbumId(@Param("objectId") String objectId, @Param("albumId") Long albumId);

    List<MapMarker> findByWestGeoGroup(int pageSize, int itemType);
    List<MapMarker> findAllByReplyIdGroup(int pageSize);
    List<MapMarker> findByWestChina(int pageSize, int itemType);
    MarkerInfo findCommentMarkerInfo(int id);
    MarkerInfo findMarkerInfo(int id);
    MarkerInfo findAlbumMarkerInfo(int id);
}
